(*
% This file defines functions which return the values of the expected value function
% and of the controls as a function of the state variables at any point in time using
% the approximate functions created by interpolation. 
% The following are defined- 
%	EtVtp1Raw
%	EtVtp1
%	Vt		
%	Choicet - the homeownership choice
%	CtOpt	- non-housing consumption
%	ZtOpt	- housing services consumption
%	htOpt	- house value 
%	stOpt 	- savings 
*)

(* % Calculate the expected valu of Vtp1 *)
ClearAll[EtVtp1Raw];
EtVtp1Raw[st_,htEnd_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] :=
Sum[           (* over possible future employment states              *)
 Sum[          (* over possible future aggregate states               *)
  Sum[         (* over possible future values of the permanent shock  *)
   Sum[        (* over possible future values of the transitory shock *)
    (epVals[[LoopOverAggStates,LoopOverEmpStates,LoopOverepVals]] G[[LifePosn+1,LoopOverAggStates]])^(1-rho) *
     Vt[
        {
         (R[[LoopOverAggStates]]/(epVals[[LoopOverAggStates,LoopOverEmpStates,LoopOverepVals]] G[[LifePosn+1,LoopOverAggStates]])) st
             +etVals[[LoopOverAggStates,LoopOverEmpStates,LoopOveretVals]]
        ,(htEnd*(1-delta))/(G[[LifePosn+1,LoopOverAggStates]] epVals[[LoopOverAggStates,LoopOverEmpStates,LoopOverepVals]])
        ,LoopOverAggStates
        ,DebtRatioWhenBoughtState
        ,DebtRatioOnNewHomesState
        ,LoopOverEmpStates
        ,LifePosn+1
        }
       ]                                                                    (* end Vt *)
     *etProb[[LoopOverAggStates,LoopOverEmpStates,LoopOveretVals]]
   ,{LoopOveretVals,Length[etVals[[LoopOverAggStates,LoopOverEmpStates]]]}] (* End loop over etVals    *)
    *epProb[[LoopOverAggStates,LoopOverEmpStates,LoopOverepVals]]
  ,{LoopOverepVals,Length[epVals[[LoopOverAggStates,LoopOverEmpStates]]]}]  (* End loop over epVals    *)
   *AggStateProb[AggState,LoopOverAggStates]
 ,{LoopOverAggStates,NumOfAggStates}]                                       (* End loop over AggStates *)
 *EmpStateProb[AggState,EmpState,LoopOverEmpStates]
,{LoopOverEmpStates,NumOfEmpStates}];                                       (* End loop over EmpStates *)

EtVtp1Raw[OutStateList_] := EtVtp1Raw[OutStateList[[stPos]]
                                     ,OutStateList[[htEndPos]]
                                     ,OutStateList[[AggStatePos]]
                                     ,OutStateList[[DebtRatioWhenBoughtStatePos]]
                                     ,OutStateList[[DebtRatioOnNewHomesStatePos]]
                                     ,OutStateList[[EmpStatePos ]]
                                     ,OutStateList[[LifePosnPos ]]
                                     ];


ClearAll[EtVtp1];
EtVtp1[st_,htEnd_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := Block[{},
sarg=st;
If[st>smax,sarg=smax+(sarg-smax)/2];
harg=htEnd;
If[harg>hsupermax,harg=hsupermax+(harg-hsupermax)/2];
If[harg<hmin,harg=hmin];
If[sarg<smin,                 (* If they are trying to save less than the minimum amount ... *)
  (* then *)
    GreatBadness,              (* Then punish them; this is how the liquidity constraint is imposed *)
  (* else *)
    If[LifePosn<LifeLength,   (* If this is not the last period of life *)
      (* then *)              (* Then return the interpolation so long as it is negative, otherwise zero *)
        Min[EtVtp1Interp[sarg,harg,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState,LifePosn],0], 
      (* else *) 0            (* This is the last period of life, so the value of saving is zero *)
      ]
  ]
];

EtVtp1[OutStateList_]    :=    EtVtp1[OutStateList[[stPos]]
                                     ,OutStateList[[htEndPos]]
                                     ,OutStateList[[AggStatePos]]
                                     ,OutStateList[[DebtRatioWhenBoughtStatePos]]
                                     ,OutStateList[[DebtRatioOnNewHomesStatePos]]
                                     ,OutStateList[[EmpStatePos ]]
                                     ,OutStateList[[LifePosnPos ]]
                                     ];

EtVtp1Interp[st_,htEnd_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_]:= 
  EtVtp1InterpFunc[[LifePosn]][st,htEnd,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState];

Clear[VtInterp];
VtInterp[xt_,htStart_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := 
  VtInterpFunc[[LifePosn]][xt,htStart,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState];
			  

Clear[Vt];
Vt[xt_,htStart_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := 
Block[{},
If[xt   < xmin,      (* If they have arrived in this period with negative assets *)
  GreatBadness,      (* then punish them                                         *)
  (* else *)         (* else proceed normally                                    *)
  xarg=xt;
  If[xt>xmax,xarg=xmax+(xarg-xmax)/2];
  harg=htStart;
  If[harg>hsupermax,harg=hsupermax+(harg-hsupermax)/2];
  PeriodOfOptimalRule = LifePosn;
  If[LifePosn < EarliestPeriodSolved,PeriodOfOptimalRule = EarliestPeriodSolved]; 
  If[LifePosn==LifeLength,                           (* If it's the last period of life                                      *)
    (* then *)                                       (* then sell the house and consume optimal Cobb-Douglas shares          *)
       w = xt + harg((1-DebtRatioGrid[[DebtRatioWhenBoughtState]])-b);
       u[(1-alpha)w,alpha w / lambda[[AggState]] q],
    (* else *)                                       (* else return the interpolation                                        *)
       Min[0,VtInterp[xarg,harg,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState,LifePosn]]
    ] (* End If[LifePosn==LifeLength] *)
  ]   (* End If[xt < xmin]            *)
]; (* End of Block *)

Vt[listinput_] := Vt[
                               listinput[[xtPos]]
                              ,listinput[[htStartPos]]
                              ,listinput[[AggStatePos]]
                              ,listinput[[DebtRatioWhenBoughtStatePos]]
                              ,listinput[[DebtRatioOnNewHomesStatePos]]
                              ,listinput[[EmpStatePos]]
                              ,listinput[[LifePosnPos]]
                              ];

ClearAll[ChoicetInterp];
ChoicetInterp[xt_,htStart_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := 
ChoicetInterpFunc[[1]][xt,htStart,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState];

ClearAll[Choicet];
Choicet[xt_,htStart_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := Block[{},
  xarg=xt;
  If[xt>xmax,xarg=xmax+(xarg-xmax)/2];
  harg=htStart;
  If[harg>hsupermax,harg=hsupermax+(harg-hsupermax)/2];
  PeriodOfOptimalRule = LifePosn;
  If[LifePosn < EarliestPeriodSolved,PeriodOfOptimalRule = EarliestPeriodSolved];
  ChoicetInterp[xarg,harg,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState,PeriodOfOptimalRule]];

Choicet[listinput_] := Choicet[
                               listinput[[xtPos]]
                              ,listinput[[htStartPos]]
                              ,listinput[[AggStatePos]]
                              ,listinput[[DebtRatioWhenBoughtStatePos]]
                              ,listinput[[DebtRatioOnNewHomesStatePos]]
                              ,listinput[[EmpStatePos]]
                              ,listinput[[LifePosnPos]]
                              ];


ClearAll[ctOpt];
ctOpt[xt_,htStart_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := Block[{},
  xarg=xt;
  If[xt>xmax,xarg=xmax+(xarg-xmax)/2];
  harg=htStart;
  If[harg>hsupermax,harg=hsupermax+(harg-hsupermax)/2];
  PeriodOfOptimalRule = LifePosn;
  If[LifePosn < EarliestPeriodSolved,PeriodOfOptimalRule = EarliestPeriodSolved]; 
  ctOptInterp[xarg,harg,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState,PeriodOfOptimalRule]];

ctOpt[listinput_] := ctOpt[
                               listinput[[xtPos]]
                              ,listinput[[htStartPos]]
                              ,listinput[[AggStatePos]]
                              ,listinput[[DebtRatioWhenBoughtStatePos]]
                              ,listinput[[DebtRatioOnNewHomesStatePos]]
                              ,listinput[[EmpStatePos]]
                              ,listinput[[LifePosnPos]]
                              ];

ClearAll[ctOptInterp];
ctOptInterp[xt_,htStart_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := 
  ctOptInterpFunc[[LifePosn]][xt,htStart,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState];


ztOpt[xt_,htStart_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := Block[{},
  xarg=xt;
  If[xt>xmax,xarg=xmax+(xarg-xmax)/2];
  harg=htStart;
  If[harg>hsupermax,harg=hsupermax+(harg-hsupermax)/2];
  PeriodOfOptimalRule = LifePosn;
  If[LifePosn < EarliestPeriodSolved,PeriodOfOptimalRule = EarliestPeriodSolved];
  ztOptInterp[xarg,harg,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState,PeriodOfOptimalRule]];

ztOpt[listinput_] := ztOpt[
                               listinput[[xtPos]]
                              ,listinput[[htStartPos]]
                              ,listinput[[AggStatePos]]
                              ,listinput[[DebtRatioWhenBoughtStatePos]]
                              ,listinput[[DebtRatioOnNewHomesStatePos]]
                              ,listinput[[EmpStatePos]]
                              ,listinput[[LifePosnPos]]
                              ];


ClearAll[ztOptInterp];
ztOptInterp[xt_,htStart_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := 
  ztOptInterpFunc[[LifePosn]][xt,htStart,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState];


htOpt[xt_,htStart_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := Block[{},
  xarg=xt;
  If[xt>xmax,xarg=xmax+(xarg-xmax)/2];
  harg=htStart;
  If[harg>hsupermax,harg=hsupermax+(harg-hsupermax)/2];
  PeriodOfOptimalRule = LifePosn;
  If[LifePosn < EarliestPeriodSolved,PeriodOfOptimalRule = EarliestPeriodSolved];
  htOptInterp[xarg,harg,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState,PeriodOfOptimalRule]];

htOpt[listinput_] := htOpt[
                               listinput[[xtPos]]
                              ,listinput[[htStartPos]]
                              ,listinput[[AggStatePos]]
                              ,listinput[[DebtRatioWhenBoughtStatePos]]
                              ,listinput[[DebtRatioOnNewHomesStatePos]]
                              ,listinput[[EmpStatePos]]
                              ,listinput[[LifePosnPos]]
                              ];


ClearAll[htOptInterp];
htOptInterp[xt_,htStart_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := 
  htOptInterpFunc[[LifePosn]][xt,htStart,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState];


stOpt[xt_,htStart_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := Block[{},
  xarg=xt;
  If[xt>xmax,xarg=xmax+(xarg-xmax)/2];
  harg=htStart;
  If[harg>hsupermax,harg=hsupermax+(harg-hsupermax)/2];
  PeriodOfOptimalRule = LifePosn;
  If[LifePosn < EarliestPeriodSolved,PeriodOfOptimalRule = EarliestPeriodSolved];
  stOptInterp[xarg,htStart,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState,PeriodOfOptimalRule]];

stOpt[listinput_] := stOpt[
                               listinput[[xtPos]]
                              ,listinput[[htStartPos]]
                              ,listinput[[AggStatePos]]
                              ,listinput[[DebtRatioWhenBoughtStatePos]]
                              ,listinput[[DebtRatioOnNewHomesStatePos]]
                              ,listinput[[EmpStatePos]]
                              ,listinput[[LifePosnPos]]
                              ];


ClearAll[stOptInterp];
stOptInterp[xt_,htStart_,AggState_,DebtRatioWhenBoughtState_,DebtRatioOnNewHomesState_,EmpState_,LifePosn_] := 
  stOptInterpFunc[[LifePosn]][xt,htStart,AggState,DebtRatioWhenBoughtState,DebtRatioOnNewHomesState,EmpState];







