(*

This file sets up the transition matrices and other
variables which determine the nature and duration
of unemployment spells. Probabilities are based on
empirical data from NBER recessions/expansions. 
See the Employment State Section of the paper for
a more complete explanation

*)

(* % Set the employment probabilities for a consumer who is 
   % currently employed (E) *)

JobKeepHazardBoom          = .97; 
ShortUnemploymentSpellBoom = .03; 
LongUnemploymentSpellBoom  = 0.0; 

(*
JobKeepHazardBoom          = .97; (* % Prob[E->E  / BOOM] employed consumer keeps job in boom                          *)
ShortUnemploymentSpellBoom = .01; (* % Prob[E->U1 / BOOM] employed consumer switches to a 1-period unemp spell in boom *)
LongUnemploymentSpellBoom  = 	  (* % Prob[E->U2 / BOOM] employed consumer switches to a 2-period unemp spell in boom *) 	
       1- JobKeepHazardBoom-ShortUnemploymentSpellBoom;
*)

JobKeepHazardBust          = .96; (* % Prob[E->E  / BUST] employed consumer keeps job in bust                          *) 	
                                  (* % Prob[E->U1 / BUST]=0 employed consumer switches to a 1-period unemp spell in bust *)	
LongUnemploymentSpellBust  =      (* % Prob[E->U2 / BUST] employed consumer switches to a 2-period unemp spell in bust *) 	
	1- JobKeepHazardBust;

JobKeepHazardPostBust      = .96; (* % Prob[E->E  / POSTBUST] employed consumer keeps job in post bust			      *) 	
                                  (* % Prob[E->U1 / POSTBUST]=0 employed consumer switches to a 1-period unemp spell in post bust *)	
JobLoseHazardPostBust      = 	  (* % Prob[E->U2 / POSTBUST] employed consumer switches to a 2-period unemp spell in post bust *) 	
	1- JobKeepHazardPostBust;

(* % Set the employment probabilities for a consumer who is 
   % currently in a 1-period unemployment spell (U1)
   % NOTE: In our paper the employment probabilities for a consumer who is 
   % currently in a 1 period unemployment spell (U1) are the same as those 
   % for a consumer who is currently employed *)

(* % Set the employment probabilities for a consumer who is 
   % currently in the 1st period of a 2-period unemployment spell (U2) *)

ProbOfStayingUnempBoom     = 1;    	(* % Prob[U2->U2      / BOOM] consumer in the 1st period of a 2-period spell stays unemployed *)
ProbOfFindingJobBoom       = 0;         (* % Prob[U2->U1 or E / BOOM] consumer in the 1st period of a 2-period does not find a job    *)

ProbOfStayingUnempBust = 1;		(* % Prob[U2->U2      / BUST] consumer in the 1st period of a 2-period spell stays unemployed *)	
ProbOfFindingJobBust   = 0;		(* % Prob[U2->U1 or E / BUST] consumer in the 1st period of a 2-period does not find a job    *)

ProbOfStayingUnempPostBust = 1;		(* % Prob[U2->U2      / POSTBUST] consumer in the 1st period of a 2-period spell stays unemployed *)	
ProbOfFindingJobPostBust   = 0;		(* % Prob[U2->U1 or E / POSTBUST] consumer in the 1st period of a 2-period does not find a job    *)


(* % Form the overall employment state transition matrix during a boom *)
EmpStateTransitionMatrixBoom =     
{
{JobKeepHazardBoom  ,ShortUnemploymentSpellBoom,LongUnemploymentSpellBoom},(* Trans probs for someone currently employed        *)
{JobKeepHazardBoom  ,ShortUnemploymentSpellBoom,LongUnemploymentSpellBoom},(* Trans probs for person in state U_1 (see paper)   *)
{ProbOfFindingJobBoom,ProbOfStayingUnempBoom,0                            }(* Trans probs for person in state U_2 (se paper)    *)
}

(* % Form the list of cumulative probabilities during a boom *)
CumEmpStateTransitionMatrixBoom =  
{
{JobKeepHazardBoom   ,JobKeepHazardBoom+ShortUnemploymentSpellBoom,1},  (* Cum probs for someone currently employed          *)
{JobKeepHazardBoom   ,JobKeepHazardBoom+ShortUnemploymentSpellBoom,1},  (* Cum probs for person in 2nd period of unemp spell *)
{ProbOfFindingJobBoom,ProbOfStayingUnempBoom+ProbOfFindingJobBoom,1}    (* Cum probs for person in 1st period of unemp spell *)
}

(* % Form the overall employment state transition matrix during a bust *)
EmpStateTransitionMatrixBust = 
{
{JobKeepHazardBust   ,0                     ,LongUnemploymentSpellBust}, (* Trans probs for someone currently employed          *)
{JobKeepHazardBust   ,0                     ,LongUnemploymentSpellBust}, (* Trans probs for person in 2nd period of unemp spell *)
{ProbOfFindingJobBust,ProbOfStayingUnempBust,0                }    (* Trans probs for person in 1st period of unemp spell *)
}

(* % Form the list of cumulative probabilities during a bust *)
CumEmpStateTransitionMatrixBust = 
{
{JobKeepHazardBust   ,JobKeepHazardBust     ,1},                        (* Cum probs for someone currently employed          *)
{JobKeepHazardBust   ,JobKeepHazardBust     ,1},                        (* Cum probs for person in 2nd period of unemp spell *)
{ProbOfFindingJobBust,ProbOfStayingUnempBust+ProbOfFindingJobBust,1}    (* Cum probs for person in 1st period of unemp spell *)
}

(* % Form the overall employment state transition matrix during a post bust *)
EmpStateTransitionMatrixPostBust = 
{
{JobKeepHazardPostBust   ,0                     ,JobLoseHazardPostBust}, (* Trans probs for someone currently employed          *)
{JobKeepHazardPostBust   ,0                     ,JobLoseHazardPostBust}, (* Trans probs for person in 2nd period of unemp spell *)
{ProbOfFindingJobPostBust,ProbOfStayingUnempPostBust,0                }  (* Trans probs for person in 1st period of unemp spell *)
}

(* % Form the list of cumulative probabilities during a post bust *)
CumEmpStateTransitionMatrixPostBust = 
{
{JobKeepHazardPostBust   ,JobKeepHazardPostBust     ,1},                         (* Cum probs for someone currently employed  *)
{JobKeepHazardPostBust   ,JobKeepHazardPostBust     ,1},                         (* Cum probs for person in period 2 of spell *)
{ProbOfFindingJobPostBust,ProbOfStayingUnempPostBust+ProbOfFindingJobPostBust,1} (* Cum probs for person in period 1 of spell *)
}

(* % Form the overall employment state transition matrix 
   % for all aggregate states *)
EmpStateTransitionMatrix   ={   EmpStateTransitionMatrixBoom,   EmpStateTransitionMatrixBust,   EmpStateTransitionMatrixPostBust};

(* % Form the list of cumulative probabilities 
   % for all aggregate states *)
CumEmpStateTransitionMatrix={CumEmpStateTransitionMatrixBoom,CumEmpStateTransitionMatrixBust,CumEmpStateTransitionMatrixPostBust};

(* % Gives the probability of moving to the future employment state 
   % from the current employment state given the current aggregate
   % state of the economy *)
ClearAll[EmpStateProb];
EmpStateProb[AggState_,EmpStateCurrent_,EmpStateFuture_] := 
  EmpStateTransitionMatrix[[AggState,EmpStateCurrent,EmpStateFuture]];

CumEmpStateNext[AggState_,EmpStateCurrent_,RandomDraw_] := 
  FirstElementGreaterThan[CumEmpStateTransitionMatrix[[AggState,EmpStateCurrent]],RandomDraw];

EmpStateEmployed   = 1;
EmpStateUnemployed = 2;
EmpStateJobLost    = 3;
NumOfEmpStates = 3;


(* % Define permanent and transitory income growth shocks in BOOMS *)

(* % Define the distribution of permanent income shocks for a consumer 
   % who is employed (E) *)
(*epValsEmployedBoom      = {0.95,1.00,1.05}^(2/PeriodsPerYear);*)
epValsEmployedBoom      = {0.95,1.00,1.05};
epProbEmployedBoom      = {0.33,0.34,0.33};
epValsEmployedBoomMean  = epValsEmployedBoom . epProbEmployedBoom;
epProbEmployedBoomCum   = CumulativeSums[epProbEmployedBoom];

(* % Define the distribution of permanent income shocks for a consumer 
   % who is in a 1-period unemployment spell (U1) *)
epValsUnemployedBoom    = {0.8 ,0.90,1.0};
epProbUnemployedBoom    = {0.33,0.34,0.33};
epValsUnemployedBoomMean= epValsUnemployedBoom . epProbUnemployedBoom;
epProbUnemployedBoomCum = CumulativeSums[epProbUnemployedBoom];

(* % During the first period of a two-period spell (U2), i.e., 
   % if status is JobLost, leave permanent income alone *)
epValsJobLostBoom       = {1.};
epProbJobLostBoom       = {1.};
epValsJobLostBoomMean   = epValsJobLostBoom . epProbJobLostBoom;
epProbJobLostBoomCum    = CumulativeSums[epProbJobLostBoom];

(* % NOTE: In our paper, we keep transitory shocks the same 
   % in all aggregate states *)

(* % Define the distribution of transitory income shocks for a consumer 
   % who is employed (E) *)
etValsEmployedBoom      = {0.9 ,1.00,1.10};
etProbEmployedBoom      = {0.33,0.34,0.33};
etValsEmployedBoomMean  = etValsEmployedBoom . etProbEmployedBoom;
etProbEmployedBoomCum   = CumulativeSums[etProbEmployedBoom];

(* % Define the distribution of transitory income shocks for a consumer 
   % who is in a 1-period unemployment spell (U1) *)
etValsUnemployedBoom    = {0.5};
etProbUnemployedBoom    = {1.0};
etValsUnemployedBoomMean= etValsUnemployedBoom . etProbUnemployedBoom;
etProbUnemployedBoomCum = CumulativeSums[etProbUnemployedBoom];

(* % Define the distribution of transitory income shocks for a consumer 
   % who is the 1st period of a 2-period unemployment spell (U2) *)
etValsJobLostBoom       = etValsUnemployedBoom;
etProbJobLostBoom       = etProbUnemployedBoom;
etValsJobLostBoomMean   = etValsJobLostBoom . etProbJobLostBoom;
etProbJobLostBoomCum    = CumulativeSums[etProbJobLostBoom];


(* % Define permanent and transitory income growth shocks in BUSTS *)

(* % Define the distribution of permanent income growth shocks for a consumer 
   % who is employed (E) *)
(* NOTE: In the absence of any good data for calibration, we 
   keep the size of permanent shocks the same for employed people
   in a bust as in a boom *)
epValsEmployedBust      = epValsEmployedBoom;
epProbEmployedBust      = epProbEmployedBoom;
epValsEmployedBustMean  = epValsEmployedBust . epProbEmployedBust;
epProbEmployedBustCum   = CumulativeSums[epProbEmployedBust];

(* % Define the distribution of permanent income shocks for a consumer 
   % who is in a 1-period unemployment spell (U1) *)
(* NOTE: Capture the assumption that unemployment in a bust is worse 
   than in a boom by assuming that permanent income shocks are larger 
   in a bust *)
epValsUnemployedBust    = {0.7 ,0.90,1.1};
epProbUnemployedBust    = {0.33,0.34,0.33}; 
epValsUnemployedBustMean= epValsUnemployedBust . epProbUnemployedBust;
epProbUnemployedBustCum = CumulativeSums[epProbUnemployedBust];

(* % During the first period of a two-period spell (U2), i.e., 
   % if status is JobLost, leave permanent income alone *)
epValsJobLostBust    = {1.};
epProbJobLostBust    = {1.};
epValsJobLostBustMean= epValsJobLostBust . epProbJobLostBust;
epProbJobLostBustCum = {1.};

(* % NOTE: In our paper, we keep transitory shocks the same 
   % in all aggregate states *)

(* % Define the distribution of transitory income shocks for a consumer 
   % who is employed (E) *)
etValsEmployedBust      = etValsEmployedBoom;
etProbEmployedBust      = etProbEmployedBoom;(*{0.33 ,0.34,0.33};*)
etValsEmployedBustMean  = etValsEmployedBust . etProbEmployedBust;
etProbEmployedBustCum   = CumulativeSums[etProbEmployedBust];

(* % Define the distribution of transitory income shocks for a consumer 
   % who is in a 1-period unemployment spell (U1) *)
etValsUnemployedBust    = etValsUnemployedBoom;
etProbUnemployedBust    = etProbUnemployedBoom;
etValsUnemployedBustMean= etValsUnemployedBust . etProbUnemployedBust;
etProbUnemployedBustCum = etProbUnemployedBoomCum;

(* % Define the distribution of transitory income shocks for a consumer 
   % who is the 1st period of a 2-period unemployment spell (U2) *)
etValsJobLostBust       = etValsUnemployedBust;
etProbJobLostBust       = etProbUnemployedBust;
etValsJobLostBustMean   = etValsJobLostBust . etProbJobLostBust;
etProbJobLostBustCum    = etProbUnemployedBustCum;


(* % Define permanent and transitory income shocks in POST BUSTS *)
(* % NOTE: In our paper, we keep both permanent and transitory income shocks
   % the same in PostBust periods as in Bust periods *)

(* % Define the distribution of permanent income growth shocks for a consumer 
   % who is employed (E) *)
epValsEmployedPostBust      = epValsEmployedBoom;
epProbEmployedPostBust      = epProbEmployedBoom;(*{0.33,0.34,0.33};*)
epValsEmployedPostBustMean  = epValsEmployedPostBust . epProbEmployedPostBust;
epProbEmployedPostBustCum   = CumulativeSums[epProbEmployedPostBust];

(* % Define the distribution of permanent income shocks for a consumer 
   % who is in a 1-period unemployment spell (U1) *)
epValsUnemployedPostBust    = epValsUnemployedBust;
epProbUnemployedPostBust    = epProbUnemployedBust;
epValsUnemployedPostBustMean= epValsUnemployedPostBust . epProbUnemployedPostBust;
epProbUnemployedPostBustCum = CumulativeSums[epProbUnemployedPostBust];

(* % During the first period of a two-period spell (U2), i.e., 
   % if status is JobLost, leave permanent income alone *)
epValsJobLostPostBust    = {1.};
epProbJobLostPostBust    = {1.};
epValsJobLostPostBustMean= epValsJobLostPostBust . epProbJobLostPostBust;
epProbJobLostPostBustCum = {1.};

(* % NOTE: In our paper, we keep transitory shocks the same 
   % in all aggregate states *)

(* % Define the distribution of transitory income shocks for a consumer 
   % who is employed (E) *)
etValsEmployedPostBust      = etValsEmployedBoom;(*{0.9 ,1.00,1.10};*)
etProbEmployedPostBust      = etProbEmployedBoom;(*{0.33 ,0.34,0.33};*)
etValsEmployedPostBustMean  = etValsEmployedPostBust . etProbEmployedPostBust;
etProbEmployedPostBustCum   = CumulativeSums[etProbEmployedPostBust];

(* % Define the distribution of transitory income shocks for a consumer 
   % who is in a 1-period unemployment spell (U1) *)
etValsUnemployedPostBust    = etValsUnemployedBoom;
etProbUnemployedPostBust    = etProbUnemployedBoom;
etValsUnemployedPostBustMean= etValsUnemployedPostBust . etProbUnemployedPostBust;
etProbUnemployedPostBustCum = etProbUnemployedBoomCum;

(* % Define the distribution of transitory income shocks for a consumer 
   % who is the 1st period of a 2-period unemployment spell (U2) *)
etValsJobLostPostBust       = etValsUnemployedPostBust;
etProbJobLostPostBust       = etProbUnemployedPostBust;
etValsJobLostPostBustMean   = etValsJobLostPostBust . etProbJobLostPostBust;
etProbJobLostPostBustCum    = etProbUnemployedPostBustCum;

(* % Define the list of transitory income shock values in booms *)
etValsBoom    = {etValsEmployedBoom   ,etValsUnemployedBoom   ,etValsJobLostBoom   };
(* % Define the list of transitory income shock probabilities in booms *)
etProbBoom    = {etProbEmployedBoom   ,etProbUnemployedBoom   ,etProbJobLostBoom   };
etProbBoomCum = {etProbEmployedBoomCum,etProbUnemployedBoomCum,etProbJobLostBoomCum};

(* % Define the list of permanent income shock values in booms *)
epValsBoom    = {epValsEmployedBoom   ,epValsUnemployedBoom   ,epValsJobLostBoom   };
(* % Define the list of permanent income shock probabilities in booms *)
epProbBoom    = {epProbEmployedBoom   ,epProbUnemployedBoom   ,epProbJobLostBoom   };
epProbBoomCum = {epProbEmployedBoomCum,epProbUnemployedBoomCum,epProbJobLostBoomCum};

(* % Define the list of transitory income shock values in busts *)
etValsBust    = {etValsEmployedBust   ,etValsUnemployedBust   ,etValsJobLostBust   };
(* % Define the list of transitory income shock probabilities in busts *)
etProbBust    = {etProbEmployedBust   ,etProbUnemployedBust   ,etProbJobLostBust   };
etProbBustCum = {etProbEmployedBustCum,etProbUnemployedBustCum,etProbJobLostBustCum};

(* % Define the list of permanent income shock values in busts *)
epValsBust    = {epValsEmployedBust   ,epValsUnemployedBust   ,epValsJobLostBust   };
(* % Define the list of permanent income shock probabilities in busts *)
epProbBust    = {epProbEmployedBust   ,epProbUnemployedBust   ,epProbJobLostBust   };
epProbBustCum = {epProbEmployedBustCum,epProbUnemployedBustCum,epProbJobLostBustCum};

(* % Define the list of transitory income shock values in postbusts *)
etValsPostBust    = {etValsEmployedPostBust   ,etValsUnemployedPostBust   ,etValsJobLostPostBust   };
(* % Define the list of transitory income shock probabilities in postbusts *)
etProbPostBust    = {etProbEmployedPostBust   ,etProbUnemployedPostBust   ,etProbJobLostPostBust   };
etProbPostBustCum = {etProbEmployedPostBustCum,etProbUnemployedPostBustCum,etProbJobLostPostBustCum};

(* % Define the list of permanent income shock values in postbusts *)
epValsPostBust    = {epValsEmployedPostBust   ,epValsUnemployedPostBust   ,epValsJobLostPostBust   };
(* % Define the list of permanent income shock probabilities in postbusts *)
epProbPostBust    = {epProbEmployedPostBust   ,epProbUnemployedPostBust   ,epProbJobLostPostBust   };
epProbPostBustCum = {epProbEmployedPostBustCum,epProbUnemployedPostBustCum,epProbJobLostPostBustCum};

(* % Define the list of all transitory income shock values *)
etVals = {etValsBoom,etValsBust,etValsPostBust};
(* % Define the list of all permanent income shock values *)
epVals = {epValsBoom,epValsBust,epValsPostBust};

(* % Define the list of all transitory income shock probabilities *)
etProb = {etProbBoom,etProbBust,etProbPostBust};
etProbCum = {etProbBoomCum,etProbBustCum,etProbPostBustCum};
(* % Define the list of all permanent income shock probabilities *)
epProb = {epProbBoom,epProbBust,epProbPostBust};
epProbCum = {epProbBoomCum,epProbBustCum,epProbPostBustCum};

